function ss = gibbs_category_ibp( x , param_set )
% ss = gibbs_category_ibp( x , param_set )
[ sample_count dim_count ] = size( x );
z_set = []; c_set = []; m_set = []; ll_set = []; lj_set = []; a_set = []; a = []; 
integrate_new_features = true;

% initialise hidden variables
[z c m] = generate_category_ibp( sample_count , param_set );
switch param_set.model_type
    case 'lg'
        a = param_set.mean_f + param_set.sigma_f * randn( size( z , 2 ) , dim_count );
    case 'exp'
        a = random( 'exp' , param_set.lambda , [ size( z , 2 ) dim_count ] );
    case 'detector'
        while size( z , 2 ) < dim_count
            [z c m] = generate_category_ibp( sample_count , param_set );
        end
        z = z( : , 1:dim_count );
        m = m( : , 1:dim_count );
end

if isfield( param_set , 'cheat' )
   c = param_set.cheat_c;
   m = param_set.cheat_m;
   z = param_set.cheat_z;
   a = param_set.cheat_a;
   feature_count = param_set.cheat_feature_count;
end

% loop and sample
for iter = 1:param_set.iter_count
    disp(sprintf(' --- starting iter %d/%d with %d categories and %d features' , ...
        iter , param_set.iter_count , size( c , 2 ) , size( m , 2 ) ));

    % resample z
    pz0 = param_set.q.^( c * m );
    lprior0 = log( pz0 );
    lprior1 = log( 1 - pz0 );
    for n = randperm( sample_count )
        for k = randperm( size( z , 2 ) )
            z( n , k ) = 0;
            ll0 = ldata( x( n , : ) , z( n , : ) , a , param_set , n );
            z( n , k ) = 1;
            ll1 = ldata( x( n , : ) , z( n , : ) , a , param_set , n );
            z( n , k ) = sampleLMultinomial( [ ( lprior0( n , k ) + ll0 ) ( lprior1( n , k ) + ll1 ) ] ) - 1;
            
            % do an additional mh, to encourage categories to stay on
            if ll1 > ll0 && isinf( lprior1( n , k ) )
                cat_ind = find( m( : , k ) == 1 , 1 );
                if ~isempty( cat_ind )
                    lprior_cat = compute_ibp_prior( c , n , cat_ind );
                    z( n , k ) = 0; c( n , cat_ind ) = 0;
                    lz0 = zlik( z( n , : ) , c( n , : ) , m , param_set );
                    z( n , k ) = 1; c( n , cat_ind ) = 1;
                    lz1 = zlik( z( n , : ) , c( n , : ) , m , param_set );
                    ll = lprior_cat + [ lz0 lz1 ] + [ ll0 ll1 ];
                    if sampleLMultinomial( ll ) == 1
                        z( n , k ) = 0; c( n , cat_ind ) = 0;
                    else
                        z( n , k ) = 1; c( n , cat_ind ) = 1;
                    end                
                end
            end            
        end
    end
    
    % resample a
    a = resample_a( x , z , param_set );

    % resample c
    for n = 1:size( c , 1 )
        
        % existing categories
        for k = 1:size( c , 2 )
            lprior = compute_ibp_prior( c , n , k );
            c( n , k ) = 0;
            ll0 = zlik( z( n , : ) , c( n , : ) , m , param_set );
            c( n , k ) = 1;
            ll1 = zlik( z( n , : ) , c( n , : ) , m , param_set );
            
            % sample, ensuring that z stays valid
            if isinf( lprior(2) )
                c( n , k ) = 0;
                emp_z1_ind = find( z( n , : ) == 1 );
                mod_z0_ind = find( c( n , : ) * m == 0 );
                z( n , intersect( emp_z1_ind , mod_z0_ind ) ) = 0;
            else
                c( n , k ) = sampleLMultinomial( lprior + [ ll0 ll1 ] ) - 1;
            end
        end
        
        % new categories
        k_new = poissrnd( param_set.alpha_c / sample_count );
        if k_new > 0 
            ll0 = zlik( z( n , : ) , c( n , : ) , m , param_set ) + ...
                ldata( x( n , : ) , z( n , : ) , a , param_set , n );
            [ m_new a_new ] = extend_m( m , a , k_new , param_set );
            z_new = z( n , : );
            f_new = size( m_new , 2 ) - size( m , 2 );
            if f_new > 0
                z_new = [ z_new double( rand( 1 , f_new ) < ...
                    ( 1 - param_set.q.^( sum( m_new( : , (end-f_new+1):end ) , 1 ) ) ) ) ];
            end
            ll1 = zlik( z_new , [ c( n , : ) ones( 1 , k_new ) ] , m_new , param_set ) + ...
                ldata( x( n , : ) , z_new , a_new , param_set , n );
                
            % sample
            if sampleLMultinomial( [ ll0 ll1 ] ) == 2
                c( n , (end+1):(end+k_new) ) = 1;
                m = m_new;
                z = [ z zeros( sample_count , f_new ) ];
                z( n , : ) = z_new;
                a = resample_a( x , z , param_set );
                disp(sprintf( 'on data %d, %d new categories for total of %d' , ...
                    n , k_new , size( c , 2 ) ));
            end
        end
    end
    
    % resample m
    for k = 1:size( m , 1 )
        k_ind = find( c( : , k ) == 1 );
        x_k = x( k_ind , : );
        c_k = c( k_ind , : );
        z_k = z( k_ind , : );
        
        % existing features
        for j = 1:size( m , 2 )
            hack_inf = strcmp( 'detector' , param_set.model_type );
            lprior = compute_ibp_prior( m , k , j , hack_inf );
            m( k , j ) = 0;
            ll0 = zlik( z_k , c_k , m , param_set );
            m( k , j ) = 1;
            ll1 = zlik( z_k , c_k , m , param_set );
            
            % sample, ensuring z stays valid
            if isinf( lprior(2) )
                m( k , j ) = 0;
                emp_z1_ind = find( z_k( : , j ) == 1 );
                mod_z0_ind = find( c_k * m( : , j ) == 0 );
                z_k( intersect( emp_z1_ind , mod_z0_ind ) , j ) = 0;
                z( k_ind , j ) = z_k( : , j );
            else
                m( k , j ) = sampleLMultinomial( lprior + [ ll0 ll1 ] ) - 1;
            end
        end
        
        
        % adding new features - if we integrate new features, only choose
        % to add one or none, as we cannot integrate more than that for the
        % exponential likelihood (multidimensional correlated erf, eep!)
        switch param_set.model_type
            case { 'lg' , 'exp' },
                ll0 = 0; ll1 = -Inf;
                if integrate_new_features
                    k_new = rand > .5;
                    if k_new > 0
                        ll0 = log( poisspdf( 0 , param_set.alpha_e / size( m , 1 ) ) ) ...
                            + ldata( x_k , z_k , a , param_set , k_ind );
                        z_new = [ z_k double( rand( size( z_k , 1 ) , k_new ) < ( 1 - param_set.q ) ) ];
                        ll1 = log( poisspdf( 1 , param_set.alpha_e / size( m , 1 ) ) ) ...
                            + ldata( x_k , z_new , a , param_set , k_ind );
                    end
                else
                    p_poisson = poisspdf( 0:5 , param_set.alpha_e / size( m , 1 ) );
                    p_adjust = p_poisson; p_adjust( 2 ) = p_poisson( 2 ) + 1;
                    k_new = randsample( 0:5 , 1 , true , p_adjust );
                    % k_new = poissrnd( param_set.alpha_e / size( m , 1 ) );
                    if k_new > 0
                        ll0 = ldata( x_k , z_k , a , param_set , k_ind );
                        z_new = [ z_k double( rand( size( z_k , 1 ) , k_new ) < ( 1 - param_set.q ) ) ];
                        switch param_set.model_type
                            case 'lg', a_new = [ a ; param_set.mean_f + param_set.sigma_f * randn( k_new , dim_count ) ];
                            case 'exp', a_new = [ a ; random( 'exp' , param_set.lambda , [ k_new , dim_count ] ) ];
                        end

                        % finale! debuggering! -- specific to exp ll!
                        % z_tmp = [ z zeros( sample_count , k_new ) ];
                        % z_tmp( k_ind , : ) = z_new;
                        % a_new = resample_a( x , z_tmp , param_set );
                        % lprior = [ log( p_poisson( 1 ) / p_adjust( 1 ) ) log( p_poisson( k_new + 1 ) / p_adjust( k_new + 1 ) ) ];
                        % lprior = lprior + [ lpdfExp( a , param_set.lambda ) - lpdfRNormal( a , x , z , param_set ) ...
                        %    lpdfExp( a_new , param_set.lambda ) - lpdfRNormal( a_new , x , z_tmp , param_set ) ];

                        ll1 = ldata( x_k , z_new , a_new , param_set , k_ind );
                    end
                end

                % sample
                if sampleLMultinomial( [ ll0 ll1 ] ) == 2
                    m( k , (end+1):(end+k_new) ) = 1;
                    z = [ z zeros( sample_count , k_new ) ];
                    z( k_ind , : ) = z_new;
                    a = resample_a( x , z , param_set );
                    disp(sprintf( 'on cat %d, %d new features for total of %d' , ...
                        k , k_new , size( m , 2 ) ));
                end
        end
    end

    % get rid of unused stuffs
    [ c m z a ] = clear_unused( c , m , z , a , param_set );

    % resample hyperparameters
    if param_set.resample_hyper_set
        switch param_set.model_type
            case { 'lg' , 'exp' },
                param_set.sigma_f = sqrt( 1 / random( 'gam' , 1 + numel( a ) , 1 / ...
                    ( 1 + 1/2 * sum(sum( a.^2 )) ) ));
                param_set.sigma_n = sqrt( 1 / random( 'gam' , 1 + numel( x ) , 1 / ...
                    ( 1 + 1/2 * sum(sum( ( x - z * a ).^2 )) ) ));
                param_set.lambda = random( 'gam' , 1 + numel( a ) , 1 / ( 1 + sum( a(:) ) ) );
        end
        param_set.alpha_e = random( 'gam' , 1 + sum( m(:) ) , 1 / ( 1 + size( m , 1 ) ) );
        param_set.alpha_c = random( 'gam' , 1 + sum( c(:) ) , 1 / ( 1 + sample_count ) );
    end

    % compute ll
    ll = ldata( x , z , a , param_set , 1:sample_count );
    lj = ll + compute_lprior( z , c , m , a , param_set );

    % store stuffs
    ss.h_set( iter , : ) = [ param_set.sigma_f param_set.sigma_n param_set.lambda param_set.alpha_e param_set.alpha_c ];
    ss.z_set{ iter } = z;
    ss.m_set{ iter } = m;
    ss.c_set{ iter } = c;
    ss.a_set{ iter } = a;
    ss.ll_set( iter ) = ll;
    ss.lj_set( iter ) = lj;
    save tmp_gibbs_cat_ibp

end
disp('done with inference')


% ---------------------------------------------------------------------- %
function lz = zlik( z , c , m , param_set )
z0_ind = find( z == 0 );
z1_ind = find( z == 1 );

% compute lprobabilities and combine
pz0 = param_set.q.^( c * m );
lpz0 = log( pz0 );
lpz1 = log( 1 - pz0 );
lz = sum(sum( lpz0( z0_ind ) )) + sum(sum( lpz1( z1_ind ) ));

% ----------------------------------------------------------------------- %
function ll = compute_lprior( z , c , m , a , param_set )
z1_ind = find( z == 1 ); z0_ind = find( z == 0 );
pz0 = param_set.q.^( c * m );
lz0 = log( pz0 ); lz1 = log( 1 - pz0 );
ll = lpdfIBP( c , param_set.alpha_c , 1 ) ...
    + lpdfIBP( m , param_set.alpha_e , 1 ) ...
    + sum( lz0( z0_ind ) ) + sum( lz1( z1_ind ) );
switch param_set.model_type
    case 'lg' , ll = ll + lpdfNormal( a , param_set.mean_f , param_set.sigma_f );
    case 'exp' , ll = ll + lpdfExp( a , param_set.lambda );
end



