function ss = gibbs_dp_ibp( x , param_set )
% function ss = gibbs_dp_ibp( x , param_set )
[ sample_count dim_count ] = size( x );
z_set = []; m_set = []; c_set = []; ll_set = []; lj_set = []; a_set = []; a = [];
integrate_new_features = true;

% initialise hidden variables
[z c m] = generate_dp_ibp( sample_count , param_set );
feature_count = size( z , 2 );
switch param_set.model_type 
    case 'lg'
        a = param_set.mean_f + param_set.sigma_f * randn( feature_count , dim_count );
    case 'exp'
        a = random( 'exp' , param_set.lambda , [ feature_count , dim_count ] );
    case 'detector'
        while feature_count < dim_count
            [z c m] = generate_dp_ibp( sample_count , param_set );
            feature_count = size( z , 2 );
        end
         z = z( : , 1:dim_count );
         m = m( : , 1:dim_count );
end

if isfield( param_set , 'cheat' )
   c = param_set.cheat_c;
   m = param_set.cheat_m;
   z = param_set.cheat_z;
   a = param_set.cheat_a;
   feature_count = param_set.cheat_feature_count;
end

% loop and sample
for iter = 1:param_set.iter_count
     disp( sprintf( '---- starting iter %d with %d categories and %d features ----' , ...
        iter , size( c , 2 ) , size( m , 2 ) ) );
    
    % resample a
    a = resample_a( x , z , param_set );

    % resample m
    for k = 1:size( m , 1 )
        k_ind = find( c( : , k ) == 1 );
        x_k = x( k_ind , : );
        c_k = c( k_ind , : );
        z_k = z( k_ind , : );
        
        % existing features
        for j = 1:size( m , 2 )
            hack_inf = strcmp( 'detector' , param_set.model_type );
            lprior = compute_ibp_prior( m , k , j , hack_inf );
            z_k( : , j ) = 0;
            ll0 = ldata( x_k , z_k , a , param_set , k_ind );
            z_k( : , j ) = 1;
            ll1 = ldata( x_k , z_k , a , param_set , k_ind );
            
            % sample
            m( k , j ) = sampleLMultinomial( lprior + [ ll0 ll1 ] ) - 1;
            z_k( : , j ) = m( k , j );
        end
         
        % adding new features - if we integrate new features, only choose
        % to add one or none, as we cannot integrate more than that for the
        % exponential likelihood (multidimensional correlated erf, eep!)
        switch param_set.model_type
            case { 'lg' , 'exp' },
                ll0 = 0; ll1 = -Inf;
                if integrate_new_features
                    k_new = rand > .5;
                    if k_new > 0
                        ll0 = log( poisspdf( 0 , param_set.alpha_e / size( m , 1 ) ) ) ...
                            + ldata( x_k , z_k , a , param_set , k_ind );
                        z_new = [ z_k ones( size( z_k , 1 ) , k_new ) ];
                        ll1 = log( poisspdf( 1 , param_set.alpha_e / size( m , 1 ) ) ) ...
                            + ldata( x_k , z_new , a , param_set , k_ind );
                    end
                else
                    k_new = poissrnd( param_set.alpha_e / size( m , 1 ) );
                    if k_new > 0
                        ll0 = ldata( x_k , z_k , a , param_set , k_ind );
                        z_new = [ z_k ones( size( z_k , 1 ) , k_new ) ];
                        switch param_set.model_type
                            case 'lg', a_new = a; % a_new = [ a ; param_set.mean_f + param_set.sigma_f * randn( k_new , dim_count ) ];
                            case 'exp', a_new = [ a ; random( 'exp' , param_set.lambda , [ k_new , dim_count ] ) ];
                        end
                        ll1 = ldata( x_k , z_new , a_new , param_set , k_ind );
                    end
                end

                % sample
                if sampleLMultinomial( [ ll0 ll1 ] ) == 2
                    m( k , (end+1):(end+k_new) ) = 1;
                    z = c * m;
                    a = resample_a( x , z , param_set );
                    disp(sprintf( 'on cat %d, %d new features for total of %d' , ...
                        k , k_new , size( m , 2 ) ));
                end
        end
        z( k_ind , : ) = c_k * m;
    end
        
    % resample c, based on partial gibbs of neal 2000 mc for dpms 
    for n = randperm( sample_count )
        cn = find( c( n , : ) == 1 );
        c_count = sum( c( : , cn ) );
        
        % singleton case -- consider merging
        if c_count == 1
            p_cluster = sum( c , 1 ); p_cluster( cn ) = 0;
            new_ind = sampleMultinomial( p_cluster );
            ll_curr = ldata( x( n , : ) , m( cn , : ) , a , param_set , n );
            ll_new = ldata( x( n , : ) , m( new_ind , : ) , a , param_set , n );
            p_move = min( 1 , ( sample_count - 1 ) / param_set.alpha_c ...
                * exp( ll_new - ll_curr ) );
            if rand < p_move
                c( n , cn ) = 0;
                c( n , new_ind ) = 1;
            end

        % non-singleton -- consider becoming a singleton
        else

            % draw a new value for m 
            p_vec = sum( m , 1 ) / ( size( m , 1 ) + 1 );
            m_new = zeros( 1 , size( m , 2 ) );
            for i = 1:size( m , 2 )
                if rand < p_vec( i );
                    m_new( i ) = 1;
                end
            end
            
            % add new features for lg/exp models
            new_feature_count = 0;
            switch param_set.model_type
                case { 'lg' , 'exp' } 
                    new_feature_count = poissrnd( param_set.alpha_e / size( m , 1 ) );
                    m_new = [ m_new ones( 1 , new_feature_count ) ];
                switch param_set.model_type
                    case 'lg' , a_new = a;
                    case 'exp' , a_new = [ a ; random( 'exp' , param_set.lambda , [ new_feature_count size( a , 2 ) ] ) ];
                                    end
                case 'detector', a_new = [];
            end
            
            % compute likelihoods
            ll_curr = ldata( x( n , : ) , m( cn , : ) , a , param_set , n );
            ll_new = ldata( x( n , : ) , m_new , a_new , param_set , n );
            p_move = min( 1 , param_set.alpha_c / ( sample_count - 1 ) ...
                * exp( ll_new - ll_curr ) );
            
            % decide to keep
            if rand < p_move
                c( n , cn ) = 0;
                c( n , end + 1 ) = 1;
                m = [ m zeros( size( m , 1 ) , new_feature_count ) ; m_new ];
                z = c * m;
                switch param_set.model_type
                    case 'lg', a = resample_lg_a( x , z , param_set );
                    case 'exp', a = resample_exp_a( x , z , param_set );
                end
                disp(sprintf( ' on data %d, total count %d new_feature_count %d' , n , size( c , 2 ) , new_feature_count  ));
            end
        end
    end

    % resample c, continued: do gibbs for non-singletons
    non_singleton_category_set = find( sum( c , 1 ) > 1 );
    non_singleton_data_set = find( sum( c( : , non_singleton_category_set ) , 2 ) > 0 );
    for n = 1:length( non_singleton_data_set )
        data_ind = non_singleton_data_set( n );
        lp = log( sum( c , 1 ) );
        for i = 1:size( c , 2 )
            lp( i ) = lp( i ) + ldata( x( data_ind , : ) , m( i , : ) , a , param_set , data_ind );
        end
        cluster_ind = sampleLMultinomial( lp );
        c( data_ind , : ) = 0;
        c( data_ind , cluster_ind ) = 1;
    end
        
    % get rid of unused stuffs 
    [ c m z a ] = clear_unused( c , m , z , a , param_set );
        
    % resample hypers -- lambda formula from west, hyperparam est in dpmms
    if param_set.resample_hyper_set
        switch param_set.model_type
            case { 'lg' , 'exp' },
                param_set.sigma_f = sqrt( 1 / random( 'gam' , 1 + numel( a ) , 1 / ...
                    ( 1 + 1/2 * sum(sum( a.^2 )) ) ));
                param_set.sigma_n = sqrt( 1 / random( 'gam' , 1 + numel( x ) , 1 / ...
                    ( 1 + 1/2 * sum(sum( ( x - z * a ).^2 )) ) ));
                param_set.lambda = random( 'gam' , 1 + numel( a ) , 1 / ( 1 + sum( a(:) ) ) );
        end
        param_set.alpha_e = random( 'gam' , 1 + sum( m(:) ) , 1 / ( 1 + size( m , 1 ) ) );
        param_set.alpha_c = random( 'gam' , 1 + size( c , 2 ) - 1 , 1 / ...
            ( 1 + 0.5772 + log( sample_count ) ) );
    end

    % compute ll
    ll = ldata( x , z , a , param_set , 1:sample_count );
    lj = ll + compute_lprior( c , m , a , param_set );
    
    % store stuffs
    ss.h_set( iter , : ) = [ param_set.sigma_f param_set.sigma_n param_set.lambda param_set.alpha_e param_set.alpha_c ];
    ss.z_set{ iter } = z;
    ss.m_set{ iter } = m;
    ss.c_set{ iter } = c;
    ss.a_set{ iter } = a;
    ss.ll_set( iter ) = ll;
    ss.lj_set( iter ) = lj;
    save tmp_gibbs_dp_ibp

end
disp('done with inference')

% ----------------------------------------------------------------------- %
function ll = compute_lprior( c , m , a , param_set )
ll = lpdfDP( c , param_set.alpha_c ) + lpdfIBP( m , param_set.alpha_e );
switch param_set.model_type
    case 'lg'
        ll = ll + lpdfNormal( a , param_set.mean_f , param_set.sigma_f );
    case 'exp'
        ll = ll + lpdfExp( a , param_set.lambda );
end





