function ss = gibbs_ibp_ibp( x , param_set )
% ss = gibbs_ibp_ibp( x , param_set )
[ sample_count dim_count ] = size( x );
z_set = []; m_set = []; c_set = []; ll_set = []; lj_set = []; a_set = []; a = [];
integrate_new_features = true;

% initialise assignments from the prior
param_set.q = 1;
[z c m] = generate_category_ibp( sample_count , param_set );
while size( z , 2 ) == 0
    [z c m] = generate_category_ibp( sample_count , param_set );
end
switch param_set.model_type
    case 'lg'
        a = param_set.mean_f + param_set.sigma_f * randn( size( z , 2 ) , dim_count );
    case 'exp'
        a = random( 'exp' , param_set.lambda , [ size( z , 2 ) dim_count ] );
    case 'detector'
        while size( z , 2 ) < dim_count
            [z c m] = generate_category_ibp( sample_count , param_set );
        end
        z = z( : , 1:dim_count );
        m = m( : , 1:dim_count );
end

if isfield( param_set , 'cheat' )
   c = param_set.cheat_c;
   m = param_set.cheat_m;
   z = param_set.cheat_z;
   a = param_set.cheat_a;
   feature_count = param_set.cheat_feature_count;
end

% do the iters
for iter = 1:param_set.iter_count
    disp( sprintf( '---- starting iter %d with %d categories and %d features ----' , ...
        iter , size( c , 2 ) , size( m , 2 ) ) );

    % resample a
    a = resample_a( x , z , param_set );

    % resample c
    for n = 1:size( c , 1 )
        
        % existing categories
        for k = 1:size( c , 2 )
            lprior = compute_ibp_prior( c , n , k );
            c( n , k ) = 0; z_n = double( c( n , : ) * m > 0 );
            ll0 = ldata( x( n , : ) , z_n , a , param_set , n );
            c( n , k ) = 1; z_n = double( c( n , : ) * m > 0 );
            ll1 = ldata( x( n , : ) , z_n , a , param_set , n );
            
            % sample
            c( n , k ) = sampleLMultinomial( lprior + [ ll0 ll1 ] ) - 1;
        end
        z( n , : ) = double( c( n , : ) * m > 0 );
        
        % new categories
        k_new = poissrnd( param_set.alpha_c / sample_count );
        if k_new > 0 
            ll0 = ldata( x( n , : ) , z( n , : ) , a , param_set , n );
            [ m_new a_new ] = extend_m( m , a , k_new , param_set );
            f_new = size( m_new , 2 ) - size( m , 2 );
            z_new = [ z( n , : ) ones( 1 , f_new ) ];
            ll1 = ldata( x( n , : ) , z_new , a_new , param_set , n );
                
            % sample
            if sampleLMultinomial( [ ll0 ll1 ] ) == 2
                c( n , (end+1):(end+k_new) ) = 1;
                m = m_new;
                z = [ z zeros( sample_count , f_new ) ];
                z( n , : ) = z_new;
                a = resample_a( x , z , param_set );
                disp(sprintf( 'on data %d, %d new categories for total of %d' , ...
                    n , k_new , size( c , 2 ) ));
            end
        end
    end

    % resample m
    for k = 1:size( m , 1 )
        k_ind = find( c( : , k ) == 1 );
        x_k = x( k_ind , : );
        c_k = c( k_ind , : );
        z_k = z( k_ind , : );
        
        % existing features
        for j = 1:size( m , 2 )
            hack_inf = strcmp( 'detector' , param_set.model_type );
            lprior = compute_ibp_prior( m , k , j , hack_inf );
            m( k , j ) = 0; z_k = double( c_k * m > 0 ); 
            ll0 = ldata( x_k , z_k , a , param_set , k_ind );
            m( k , j ) = 1; z_k = double( c_k * m > 0 ); 
            ll1 = ldata( x_k , z_k , a , param_set , k_ind );
            
            % sample
            m( k , j ) = sampleLMultinomial( lprior + [ ll0 ll1 ] ) - 1;
            z_k( : , j ) = double( c_k * m( : , j ) > 0 ); 
        end
                 
        % adding new features - if we integrate new features, only choose
        % to add one or none, as we cannot integrate more than that for the
        % exponential likelihood (multidimensional correlated erf, eep!)
        switch param_set.model_type
            case { 'lg' , 'exp' },
                ll0 = 0; ll1 = -Inf;
                if integrate_new_features
                    k_new = rand > .5;
                    if k_new > 0
                        ll0 = log( poisspdf( 0 , param_set.alpha_e / size( m , 1 ) ) ) ...
                            + ldata( x_k , z_k , a , param_set , k_ind );
                        z_new = [ z_k ones( size( z_k , 1 ) , k_new ) ];
                        ll1 = log( poisspdf( 1 , param_set.alpha_e / size( m , 1 ) ) ) ...
                            + ldata( x_k , z_new , a , param_set , k_ind );
                    end
                else
                    p_poisson = poisspdf( 0:5 , param_set.alpha_e / size( m , 1 ) );
                    p_adjust = p_poisson; p_adjust( 2 ) = p_poisson( 2 ) + 1;
                    k_new = randsample( 0:5 , 1 , true , p_adjust );
                    % k_new = poissrnd( param_set.alpha_e / size( m , 1 ) );
                    if k_new > 0
                        ll0 = ldata( x_k , z_k , a , param_set , k_ind );
                        z_new = [ z_k ones( size( z_k , 1 ) , k_new ) ];
                        switch param_set.model_type
                             case 'lg', a_new = a; % a_new = [ a ; param_set.mean_f + param_set.sigma_f * randn( k_new , dim_count ) ];
                             case 'exp', a_new = [ a ; random( 'exp' , param_set.lambda , [ k_new , dim_count ] ) ];
                        end
    
                        % finale! debuggering! specific to exp ll!
                        % z_tmp = z; z_tmp( k_ind , end+1:end+k_new ) = 1;
                        % a_new = resample_a( x , z_tmp , param_set );
                        % lprior = [ log( p_poisson( 1 ) / p_adjust( 1 ) ) log( p_poisson( k_new + 1 ) / p_adjust( k_new + 1 ) ) ];
                        % lprior = lprior + [ lpdfExp( a , param_set.lambda ) - lpdfRNormal( a , x , z , param_set ) ...
                        %     lpdfExp( a_new , param_set.lambda ) -
                        %     lpdfRNormal( a_new , x , z_tmp , param_set ) ];
    
                        ll1 = ldata( x_k , z_new , a_new , param_set , k_ind );
                    end
                end

                % sample
                if sampleLMultinomial( [ ll0 ll1 ] ) == 2
                    m( k , (end+1):(end+k_new) ) = 1;
                    z = double( c * m > 0 );
                    z_k = z( k_ind , : );
                    a = resample_a( x , z , param_set );
                    disp(sprintf( 'on cat %d, %d new features for total of %d' , ...
                        k , k_new , size( m , 2 ) ));
                end
        end
        z( k_ind , : ) = double( c_k * m > 0 );
    end
        
    % clean up
    [ c m z a ] = clear_unused( c , m , z , a , param_set );
    
    % resample hypers
    if param_set.resample_hyper_set
        switch param_set.model_type
            case { 'lg' , 'exp' },
                param_set.sigma_f = sqrt( 1 / random( 'gam' , 1 + numel( a ) , 1 / ...
                    ( 1 + 1/2 * sum(sum( a.^2 )) ) ) );
                param_set.sigma_n = sqrt( 1 / random( 'gam' , 1 + numel( x ) , 1 / ...
                    ( 1 + 1/2 * sum(sum( ( x - z * a ).^2 )) ) ) );
                param_set.lambda = random( 'gam' , 1 + numel( a ) , 1 / ( 1 + sum( a(:) ) ) );
        end
        param_set.alpha_e = random( 'gam' , 1 + sum( m(:) ) , 1 / ( 1 + size( m , 1 ) ) );
        param_set.alpha_c = random( 'gam' , 1 + sum( c(:) ) , 1 / ( 1 + sample_count ) );
    end
    
    % compute ll
    ll = ldata( x , z , a , param_set , 1:sample_count );
    lj = ll + compute_lprior( c , m , a , param_set );
    
    % store stuffs
    ss.h_set( iter , : ) = [ param_set.sigma_f param_set.sigma_n param_set.lambda param_set.alpha_e param_set.alpha_c ];
    ss.z_set{ iter } = z;
    ss.m_set{ iter } = m;
    ss.c_set{ iter } = c;
    ss.a_set{ iter } = a;
    ss.ll_set( iter ) = ll;
    ss.lj_set( iter ) = lj;
    save tmp_gibbs_ibp_ibp

end
disp('done with inference')

% ----------------------------------------------------------------------- %
function ll = compute_lprior( c , m , a , param_set )
ll = lpdfIBP( c , param_set.alpha_c ) + lpdfIBP( m , param_set.alpha_e , 1 );
switch param_set.model_type
    case 'lg' , ll = ll + lpdfNormal( a , param_set.mean_f , param_set.sigma_f );
    case 'exp' , ll = ll + lpdfExp( a , param_set.lambda );
end


