function ll = ldata( x , z , a , param_set , x_ind_set )
% function ll = ldata( x , z , a , param_set , x_ind_set )
%   x and z are of size x_ind_set
%   x_ind_set tells param_set what subset is being used for masking
ll = 0;

% see if mask needs adjusting
if numel( x_ind_set ) == size( param_set.test_mask , 1 )
    n_ind_set = param_set.n_ind_set;    
else
    for d = 1:param_set.dim_setting_count
        [ c ia ] = intersect( x_ind_set , param_set.n_ind_set{ d } );
        n_ind_set{ d } = ia;        
    end
end

% compute ll
switch param_set.model_type
    
    % guassian model, integrates out any uninstantiated features
    case 'lg'
        feature_count = size( a , 1 );
        new_feature_count = size( z , 2 ) - feature_count;
        for d = 1:param_set.dim_setting_count
            dim_count = length( param_set.d_ind_set{ d } );
            sample_count = length( n_ind_set{ d } );
            z1 = z( n_ind_set{ d } , 1:feature_count );
            if ~isempty( z1 )
                z2 = z( n_ind_set{ d } , (feature_count+1):end );
                m = z2' * z2 + ( param_set.sigma_n / param_set.sigma_f )^2 * eye( new_feature_count );
                xc = x( n_ind_set{ d } , param_set.d_ind_set{ d } ) - z1 * a( : , param_set.d_ind_set{ d } );
                if new_feature_count > 0
                    ll = ll - dim_count * sample_count / 2 * log( 2 * pi ) ...
                        - dim_count * ( sample_count - new_feature_count ) * log( param_set.sigma_n ) ...
                        - dim_count * new_feature_count * log( param_set.sigma_f ) ...
                        - dim_count / 2 * log( det( m ) ) ...
                        - 1 / 2 / param_set.sigma_n^2 ...
                        * trace( xc' * ( eye( sample_count ) - z2 * inv( m ) * z2' ) * xc );
                else
                    ll = ll - dim_count * sample_count / 2 * log( 2 * pi * param_set.sigma_n^2 ) ...
                        - 1 / 2 / param_set.sigma_n^2 * sum( sum( xc.^2 ) );
                end
            end
        end
        
    % exponential model (still has gaussian noise, so really same as gaussian 
    % model except that we require that the a matrix to be fully instantiated)    
    case 'exp'
        new_feature_count = size( z , 2 ) - size( a , 1 );
        switch new_feature_count 
            case 0,
                for d = 1:param_set.dim_setting_count
                    dim_count = length( param_set.d_ind_set{ d } );
                    sample_count = length( n_ind_set{ d } );
                    z1 = z( n_ind_set{ d } , : );
                    if ~isempty( z1 )
                        xc = x( n_ind_set{ d } , param_set.d_ind_set{ d } ) - z1 * a( : , param_set.d_ind_set{ d } );
                        ll = ll - dim_count * sample_count / 2 * log( 2 * pi * param_set.sigma_n^2 ) ...
                            - 1 / 2 / param_set.sigma_n^2 * sum( sum( xc.^2 ) );
                    end
                end
            case 1,
                for d = 1:param_set.dim_setting_count
                    zd = z( n_ind_set{ d } , : );
                    if ~isempty( zd )
                        xd = x( n_ind_set{ d } , param_set.d_ind_set{ d } );
                        zz = zd' * zd;
                        zx = zd' * xd;
                        k = size( z , 2 );
                        not_k = 1:(k-1);
                        v = param_set.sigma_n^2 / zz( k , k );
                        m = ( param_set.lambda * param_set.sigma_n^2 + zx( k , : ) ...
                            - zz( k , not_k ) * a( not_k , param_set.d_ind_set{ d } ) ) ...
                            / zz( k , k );
                        ld = sum( log( 1 - normcdf( 0 , m , sqrt( v ) ) ) );
                        ll = ll + ld;
                    end
                end
            otherwise
                disp('ldata, exp ll: wonky! too many new features! cannot integrate!' )
        end

    % detector model    
    case 'detector'
        lz0x1 = log( param_set.xz0 );
        lz0x0 = log( 1 - param_set.xz0 );
        lz1x1 = log( param_set.xz1 );
        lz1x0 = log( 1 - param_set.xz1 );
        for d = 1:param_set.dim_setting_count
            ztmp = z( n_ind_set{ d } , param_set.d_ind_set{ d } );
            xtmp = x( n_ind_set{ d } , param_set.d_ind_set{ d } );
            z1x0 = ztmp & ( 1 - xtmp );
            z1x1 = ztmp & xtmp;
            z0x0 = ( 1 - ztmp ) & ( 1 - xtmp );
            z0x1 = ( 1 - ztmp ) & xtmp;
            z1x0_count = sum( z1x0 , 1 );
            z1x1_count = sum( z1x1 , 1 );
            z0x0_count = sum( z0x0 , 1 );
            z0x1_count = sum( z0x1 , 1 );
            ll = ll +  z0x0_count * lz0x0( param_set.d_ind_set{ d } )' ...
                + z0x1_count * lz0x1( param_set.d_ind_set{ d } )' ...
                + z1x0_count * lz1x0( param_set.d_ind_set{ d } )' ...
                + z1x1_count * lz1x1( param_set.d_ind_set{ d } )';
        end
end