function lp = lpdfRNormal( a , x , z , param_set )
% function lp = lpdfRNormal( a , x , z , param_set );
% computes the log probability density for a matrix which was drawn from a
% posterior where the prior was exponential with rate lambda and the
% likelihood was gaussian from the model x = za + noise
if any( any( a < 0 ) )
    lp = -Inf;
else
    K = size( z , 2 );
    lp = 0;
    for d_ind = 1:param_set.dim_setting_count
        zd = z( param_set.n_ind_set{ d_ind } , : );
        xd = x( param_set.n_ind_set{ d_ind } , param_set.d_ind_set{ d_ind } );
        zz = zd' * zd;
        zx = zd' * xd;
        for k = 1:K

            % mean and variance
            not_k = [ 1:k-1 k+1:K ];
            v = param_set.sigma_n^2 / zz( k , k );
            m = ( param_set.lambda * param_set.sigma_n^2 + zx( k , : ) ...
                - zz( k , not_k ) * a( not_k , param_set.d_ind_set{ d_ind } ) ) ...
                / zz( k , k );

            % compute the lp, using an approximation for the tail weight:
            % if a >> m, then the gaussian pdf looks like N(0,v) because we
            % only keep the quadratic term -- basically, far from the mean,
            % the mean doesn't matter.  the distribution for N(0,v) * h is
            % just the distribution for N(0,v)*2, a folded gaussian
            tail_weight = 1 - normcdf( 0 , m , sqrt( v ) );
            small_tail_set = find( tail_weight < .0001 );
            a_set = a( k , param_set.d_ind_set{ d_ind } );
            lp = lp - length( small_tail_set ) / 2 * log( pi * v / 2 ) ...
                - 1 / 2 / v * sum( a_set( small_tail_set ).^2 );
            
            % the rest don't need approximating
            large_tail_set = find( tail_weight >= .0001 );
            lp = lp - sum( log( tail_weight( large_tail_set ) ) ) ...
                - length( large_tail_set ) / 2 * log( 2 * pi * v ) ...
                - 1 / 2 / v * sum( ( a_set( large_tail_set ) - m( large_tail_set ) ).^2 );
        end
    end
end