function a = resample_exp_a( x , z , param_set )
% function a = resample_exp_a( x , z , param_set )
% resample a : exponential prior on a , gaussian prior on the noise
% (results in a rectified guassian) ... this is code from mikkel's website,
% http://mikkelschmidt.dk/index.php?id=22, bayesnmf.zip, for more general
% non-negative matrix factorisation problems
K = size( z , 2 );
a = zeros( K , size( x , 2 ) );

% the prior puts in an ( -lambda * a_kd ) term, the likelihood has the
% following terms:
% -- quadratic in a_kd : (Z'Z)_kk
% -- linear in a_kd : -2*(X'Z)_dk + 2*a_k'd*(Z'Z)_kk' where k' ~= k
% this gives us an overall variance of
%    v = sigma^2 / (Z'Z)_kk
% and a mean of
%    m = ( -1*(X'Z)_dk + a_k'd*(Z'Z)_kk' - lambda * sigma^2 ) / (Z'Z)_kk
% for a gaussian truncated to only positive values, the lpdf is
%  -log( 1 - C ) - 1/2log(2*pi*v) -1/2/v * ( a - m )^2
% where C = normcdf( 0 , m , sqrt( v ) )
for d_ind = 1:param_set.dim_setting_count
    zd = z( param_set.n_ind_set{ d_ind } , : );
    xd = x( param_set.n_ind_set{ d_ind } , param_set.d_ind_set{ d_ind } );

    % can only get info on features that are on!
    pos_k_ind = find( sum( zd , 1 ) > 0 );
        
    % compute sums and sample
    zz = zd' * zd;
    zx = zd' * xd;
    for k = pos_k_ind
        
        % mean and variance
        not_k = setdiff( pos_k_ind , k );
        v = param_set.sigma_n^2 / zz( k , k );
        m = ( param_set.lambda * param_set.sigma_n^2 + zx( k , : ) ...
        - zz( k , not_k ) * a( not_k , param_set.d_ind_set{ d_ind } ) ) ...
        / zz( k , k );
          
        % get the random values
        akd = randr( m , v * ones( 1 , param_set.d_count_set( d_ind ) ) );
        a( k , param_set.d_ind_set{ d_ind } ) = akd;
    end
end

% depreciated code from mikkel
% for d_ind = 1:param_set.dim_setting_count
%     zd = z( param_set.n_ind_set{ d_ind } , : );
%     xd = x( param_set.n_ind_set{ d_ind } , param_set.d_ind_set{ d_ind } );
%     E = zd' * zd;
%     F = zd' * xd;
%     for k = 1:K
%         kk = [1:k-1 k+1:K];
%         a( k , param_set.d_ind_set{ d_ind } ) = randr( ...
%             ( F(k,:) - E(k,kk)*a( kk , param_set.d_ind_set{ d_ind } ) )'/ E(k,k), ...
%             param_set.sigma_n^2 / E(k,k) , ...
%             param_set.lambda * ones( param_set.d_count_set( d_ind ) , 1 ) );
%     end
% % mikkel's code from gibbsnmf.m
% %     E = A'*A;
% %     F = A'*X;
% %     for n = 1:N
% %         if ~nB(n)
% %             nn = [1:n-1 n+1:N];
% %             B(n,:) = randr((F(n,:)-E(n,nn)*B(nn,:))'/E(n,n), ...
% %                 sigma/E(n,n), beta(n,:)');
% %         end
% %     end
% end