% SHOULD BE CALLED FROM ANOTHER SCRIPT
function tester( x , param_set )
[sample_count dim_count] = size( x );

% add paths
addpath( '../util' );
addpath( 'gen_util' );
addpath( 'inf_util' );

% create test mask
test_mask = sparse( zeros( size( x ) ) );
if sample_count < 501
    test_n_start_ind = sample_count - 9;
    test_n_middle_ind = sample_count - 5;
else
    test_n_start_ind = sample_count - 99;
    test_n_middle_ind = sample_count - 50;
end
test_mask( test_n_start_ind:test_n_middle_ind , 1:2:end ) = 1;
test_mask( (test_n_middle_ind+1):end , 2:2:end ) = 1;
param_set.test_mask = test_mask;
[ dim_setting_set tmp dim_index_set ] = unique( test_mask' , 'rows' );
param_set.dim_setting_set = dim_setting_set;
param_set.dim_setting_count = max( dim_index_set );
for ind = 1:param_set.dim_setting_count
    param_set.d_ind_set{ ind } = find( dim_index_set == ind ); 
    param_set.n_ind_set{ ind } = find( dim_setting_set( ind , : ) == 0 ); 
    param_set.d_count_set( ind ) = length( param_set.d_ind_set{ ind } );
    param_set.n_count_set( ind ) = length( param_set.n_ind_set{ ind } );
end

% ----- INFERENCE ----- %
dp_set = {}; ibp_set = {}; 
for restart = 1:param_set.restart_count

    % DP-IBP
    if param_set.do_dp_ibp
        clear ss;
        ss = gibbs_dp_ibp( x , param_set );
        ss = compute_err( x , ss , param_set );
        dp_set{ restart } = ss;
        save( strcat( param_set.fname , '_dp_inference' ) , 'x' , 'param_set' , 'dp_set' );
    end

    % IBP-IBP
    if param_set.do_ibp_ibp
        clear ss;
        ss = gibbs_ibp_ibp( x , param_set );
        ss = compute_err( x , ss , param_set );
        ibp_set{ restart } = ss;
        save( strcat( param_set.fname , '_ibp_inference' ) , 'x' , 'param_set' , 'ibp_set' );
    end
    
      % IBP-IBP
    if param_set.do_category_ibp
        clear ss;
        ss = gibbs_category_ibp( x , param_set );
        ss = compute_err( x , ss , param_set );
        cat_set{ restart } = ss;
        save( strcat( param_set.fname , '_cat_inference' ) , 'x' , 'param_set' , 'cat_set' );
    end
    
end

% ----- MAKE PLOTS ----- %
% note: this is just the most recent ll set, not all the ll's: for basic
% visualization only!  (use error for quantitative comparisons!)
if param_set.make_plots
    [ maxv maxi ] = max( ll_set );
    figure( 1 );
    subplot( 1 , 2 , 2 ); spy( m_set{ maxi } ); title( 'combo assignments' );
    figure( 2 );
    switch param_set.model_type
        case 'lg' , subplot( 1 , 3 , 3 ); imagesc( ( c_set{ maxi } * m_set{ maxi } > 0 ) * a_set{ maxi } );
        case 'detector' , subplot( 1 , 3 , 3 ); imagesc( ( c_set{ maxi } * m_set{ maxi } > 0 ) );
    end
    figure( 3 ); 
    subplot( 3 , 1 , 1 ); plot( ll_set ); title( 'joint log likelihood' )
    for i = 1:param_set.iter_count;
        category_count( i ) = size( c_set{ i } , 2 );
        feature_count( i ) = size( m_set{ i } , 2 );
    end
    subplot( 3 , 1 , 2 ); plot( category_count );
    subplot( 3 , 1 , 3 ); plot( feature_count );
end


% ----------------------------------------------------------------------- %
function ss = compute_err( x , ss_in , param_set )

% get basic stats
ss = ss_in;
train_count = sum( 1 - param_set.test_mask(:) );
test_count = sum( param_set.test_mask(:) );

% compute for several samples
for i = 1:param_set.sample_window
    switch param_set.model_type
        case { 'lg' , 'exp' }
            xpred = ss.z_set{ end - i + 1 } * ss.a_set{ end - i + 1 };
            ll_mat = -1/2 * log( 2 * pi * param_set.sigma_n^2 ) ...
                - 1/2/param_set.sigma_n^2 * ( x - xpred ).^2;
        case 'detector'
            sample_count = size( x , 1 );
            xpred = ss.z_set{ end - i + 1 };
            ll_mat = ( xpred & ( 1 - x ) ) .* repmat( 1 - param_set.xz1 , sample_count , 1 ) ...
                + ( xpred & x ) .* repmat( param_set.xz1 , sample_count , 1 ) ...
                + ( ( 1 - xpred ) & ( 1 - x ) ) .* repmat( 1 - param_set.xz0 , sample_count , 1 ) ...
                + ( ( 1 - xpred ) & x ) .* repmat( param_set.xz0 , sample_count , 1 );
    end
    err = ( x - xpred ).^2;
    train_err(i) = sum(sum( ( 1 - param_set.test_mask ) .* err )) / train_count;
    test_err(i) = sum(sum( ( param_set.test_mask ) .* err )) / test_count;
    train_ll(i) = sum(sum( ( 1 - param_set.test_mask ) .* ll_mat )) / train_count;
    test_ll(i) = sum(sum( ( param_set.test_mask ) .* ll_mat )) / test_count;
end

% average
ss.train_l2 = mean( train_err );
ss.test_l2 = mean( test_err );
ss.train_ll = mean( train_ll );
ss.test_ll = mean( test_ll );
